                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Authors note:   |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/

#include <afxwin.h>
#include <atlbase.h>
#include <afxtempl.h>

#include <GL/GL.h>
#include <GL/GLU.h>

#include "3D/Resolution.h"
#include "OpenGL/glExtensions.h"
#include "OpenGL/Texture.h"
#include "Common/CPU.h"
#include "OpenGL/Light.h"

#include "Poly.h"

#define VK_ENTER 13

/* ################################################################################################################### */

class OpenGLWindow : public CFrameWnd {
public:
    OpenGLWindow();

    void DrawWorld(bool blueish, bool aboveSurface);
    void DrawWater();
	void DrawLights(Vertex &dx, Vertex &dy);

	void DrawFrame();
protected:
	HDC hdc;
	HGLRC hrc;

	CRect windowedRect;
	int Width, Height;
	int Bpp;
	int Refresh;
	bool fullScreen;
	bool captureMouse;
	bool windowMoved;
	int middleX, middleY;

	bool Key[256];
	bool leftButton;
	float mouseSensibility;
	bool invertMouse;
	int mouseX, mouseY;

    bool showFPS;
	bool debug; // To show rendered textures
    ShaderID fragmentShader;

	Vertex position;
	float Wx, Wy, Wz;
    __int64 cpuHz;

	TextureID corona;
	Light lights[2];
	Set <Poly> polys;


	TextureHandler *texHandler;
	TextureID vectorField, renderTexture, renderTexture2, proceduralTexture;
	TextureID text, wood, woodBump, wood2, wood2Bump, wall, wallBump, ground, picture, pictureBump, groundBump;

	BOOL PreCreateWindow(CREATESTRUCT &cs);
	int  OnCreate(LPCREATESTRUCT lpCreateStruct);
	void OnSize(UINT nType, int cx, int cy);
	void OnClose();

	void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnKeyUp  (UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

	void OnLButtonDown(UINT nFlags, CPoint point);
	void OnLButtonUp  (UINT nFlags, CPoint point);
	void OnMouseMove  (UINT nFlags, CPoint point);

	void OnKillFocus(CWnd *pNewWnd);

	DECLARE_MESSAGE_MAP ()
};

BEGIN_MESSAGE_MAP(OpenGLWindow, CFrameWnd)
	
	ON_WM_CREATE ()
	ON_WM_CLOSE ()

	ON_WM_KEYDOWN ()
	ON_WM_KEYUP ()
	ON_WM_SYSKEYDOWN ()

	ON_WM_LBUTTONDOWN ()
	ON_WM_LBUTTONUP ()
	ON_WM_MOUSEMOVE ()

	ON_WM_KILLFOCUS ()

END_MESSAGE_MAP()


/* ################################################################################################################### */

void OpenGLWindow::DrawLights(Vertex &dx, Vertex &dy){
	int i, len;
	Color col;
	Vertex v, res;
	float size;

	// Draw lights
	texHandler->setBlending(GL_ONE_MINUS_DST_COLOR, GL_ONE);
	texHandler->setTextures(corona);
	texHandler->apply();

	glDepthMask(GL_FALSE);


	len = sizeof(lights) / sizeof(Light);
	for (i = 0; i < len; i++){
		col  = lights[i].getColor();
		size = 0.06f * lights[i].getSize();
		v    = lights[i].getPosition();
		glColor3fv((GLfloat *) &col);
				
		glBegin(GL_QUADS);
			res = v - size * dx - size * dy;
			glTexCoord2f(0,0);
			glVertex3fv((GLfloat *) &res);
				
			res = v + size * dx - size * dy;
			glTexCoord2f(1,0);
			glVertex3fv((GLfloat *) &res);
				
			res = v + size * dx + size * dy;
			glTexCoord2f(1,1);
			glVertex3fv((GLfloat *) &res);
				
			res = v - size * dx + size * dy;
			glTexCoord2f(0,1);
			glVertex3fv((GLfloat *) &res);
		glEnd();
	}
	glDepthMask(GL_TRUE);
}

void OpenGLWindow::DrawWorld(bool blueish, bool aboveSurface){
	int i, j, len;
	int start, stop;

	if (aboveSurface){
		start = 0;
		stop  = 17;
	} else {
		start = 17;
		stop  = polys.getSize();
	}

	len = sizeof(lights) / sizeof(Light);
	for (i = 0; i < len; i++){
		for (j = start; j < stop; j++){
			polys[j].drawLit(texHandler, lights[i], i > 0);
		}
	}
	for (i = start; i < stop; i++){
		polys[i].drawBase(texHandler, blueish);
	}

}

void OpenGLWindow::DrawWater(){
	Vertex v;
	float a;
	float i,j;

	texHandler->setFragmentShader(fragmentShader);
    texHandler->setTextures(renderTexture, renderTexture2, proceduralTexture);
    texHandler->apply();

	for (j = -512; j <= 512; j += 32){
		glBegin(GL_TRIANGLE_STRIP);
		for (i = -512; i <= 512; i += 32){
			glMultiTexCoord2f(GL_TEXTURE2_ARB, (i + 512) / 1024, (j + 512) / 1024);
			if (position.y < 0){
				a = 1;
			} else {
				v = (position - Vertex(i, 0, j));
				v.fastNormalize();
				a = v.y;
			}
			glColor4f(a,a,a,a);
			glVertex3f(i,0,j);

			glMultiTexCoord2f(GL_TEXTURE2_ARB, (i + 512) / 1024, (j + 32 + 512) / 1024);
			if (position.y < 0){
				a = 1;
			} else {
				v = (position - Vertex(i, 0, j + 32));
				v.fastNormalize();
				a = v.y;
			}
			glColor4f(a,a,a,a);
			glVertex3f(i,0,j + 32);
		}
		glEnd();
	}
}

void OpenGLWindow::DrawFrame(){
    static bool first = true;

	int i;
    static __int64 prev, curr;
    float time, speed;
    static float currTime = 0;

    prev = curr;
    curr = getCycleNumber();
    if (first){
        prev = curr;
        first = false;
    }
    time = (float) (100000.0 * double(curr - prev) / cpuHz);
	speed = time * 0.005f;

    currTime += time * 0.00003f;

    texHandler->update(proceduralTexture, (1024 * curr) / (cpuHz / 1024));

	float f = 4 * (cosf(currTime * 0.1f) + 0.5f);

	lights[0].setColor(Color(1,f,f));
	lights[0].setSize(700);
	lights[0].setPosition(Vertex(
		256 * cosf(currTime * 0.243f) * sinf(currTime * 0.341f),
		128 * cosf(currTime * 0.191f) * sinf(currTime * 0.203f) + 160,
		256 * cosf(currTime * 0.311f) * sinf(currTime * 0.172f)));
	
	lights[1].setColor(Color(f,1,f));
	lights[1].setSize(700);
	lights[1].setPosition(Vertex(
		256 * cosf(currTime * 0.341f) * sinf(currTime * 0.192f),
		128 * cosf(currTime * 0.491f) * sinf(currTime * 0.243f) + 160,
		256 * cosf(currTime * 0.265f) * sinf(currTime * 0.192f)));




	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(75, (GLdouble) Width / (GLdouble) Height, 0.01f, 2000);


    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glRotatef(Wz, 0.0f, 0.0f, 1.0f);
    glRotatef(Wx, 1.0f, 0.0f, 0.0f);
    glRotatef(Wy, 0.0f, 1.0f, 0.0f);

	glScalef(1,1,-1);

	glTranslatef(-position.x, -position.y, -position.z);


	float modelView[16];
	float projection[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,  modelView);
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	Vertex dx(modelView[0], modelView[4], modelView[8]);
	Vertex dy(modelView[1], modelView[5], modelView[9]);
	Vertex dz(modelView[2], modelView[6], modelView[10]);

	Vertex dir(0,0,0);

	if (Key[VK_LEFT ]) dir -= dx;
	if (Key[VK_RIGHT]) dir += dx;

	if (Key[VK_UP  ]) dir -= dz;
	if (Key[VK_DOWN]) dir += dz;

	if (Key[VK_SHIFT  ]) dir -= dy;
	if (Key[VK_CONTROL]) dir += dy;

	if (Key[VK_ENTER]){
		position = Vertex(384,257,-383);
		
		Wx = 30;
		Wy = -48;
		Wz = 0;
	}

	if (lengthSqr(dir) != 0){
		dir.normalize();
	}


	Vertex oldPos = position;
	position += speed * dir;


	glCullFace(GL_FRONT);
	glEnable(GL_CULL_FACE);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

    DrawWorld(position.y < 0, position.y > 0);

	glDisable(GL_CULL_FACE);

	// We don't need to update these every frame ... updating them every other frame will do
	static int p = 0;
	p++;

	if (p == 1 || (p & 1) != 0){

		texHandler->beginRenderToTexture(renderTexture);
    		glMatrixMode(GL_PROJECTION);
			glLoadMatrixf(projection);

			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();

			glRotatef(Wz, 0.0f, 0.0f, 1.0f);
			glRotatef(Wx, 1.0f, 0.0f, 0.0f);
			glRotatef(Wy, 0.0f, 1.0f, 0.0f);

			glScalef(1,1,-1);
			glTranslatef(-position.x, -position.y, -position.z);

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glEnable(GL_DEPTH_TEST);
			glDepthFunc(GL_LEQUAL);

			glCullFace(GL_FRONT);
			glEnable(GL_CULL_FACE);

			DrawWorld(false, position.y < 0);

			glDisable(GL_CULL_FACE);

			if (position.y < 0) DrawLights(dx, dy);

		texHandler->endRenderToTexture(renderTexture);
	}

	if (p == 1 || (p & 1) == 0){

		texHandler->beginRenderToTexture(renderTexture2);
    		glMatrixMode(GL_PROJECTION);
			glLoadMatrixf(projection);

			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();

			glRotatef(Wz, 0.0f, 0.0f, 1.0f);
			glRotatef(Wx, 1.0f, 0.0f, 0.0f);
			glRotatef(Wy, 0.0f, 1.0f, 0.0f);

			glScalef(1,1,-1);
			glTranslatef(-position.x, -position.y, -position.z);
			glScalef(1,-1,1);

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glEnable(GL_DEPTH_TEST);
			glDepthFunc(GL_LEQUAL);

			glCullFace(GL_BACK);
			glEnable(GL_CULL_FACE);

			DrawWorld(false, position.y > 0);

			glDisable(GL_CULL_FACE);

			if (position.y >= 0) DrawLights(dx, Vertex(dy.x, -dy.y, dy.z));

		texHandler->endRenderToTexture(renderTexture2);
	}

    static GLfloat s[4] = {1.0, 0.0, 0.0, 0.0};
    static GLfloat t[4] = {0.0, 1.0, 0.0, 0.0};
    static GLfloat r[4] = {0.0, 0.0, 1.0, 0.0};

    texHandler->setActiveTexture(0);

    glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenf(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glTexGenfv(GL_S, GL_EYE_PLANE, s);
    glTexGenfv(GL_T, GL_EYE_PLANE, t);
    glTexGenfv(GL_R, GL_EYE_PLANE, r);
    glPopMatrix();

    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);
    glEnable(GL_TEXTURE_GEN_R);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glScalef(0.5f, 0.5f, 0);
    glTranslatef(1, 1, 0);

    glMultMatrixf(projection);


	DrawWater();

    texHandler->setActiveTexture(0);

    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_R);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();


	if (position.y >= 0) DrawLights(dx, dy);

	if (showFPS){
		static float fps[8];
		char str[256];

		for (i = 0; i < 7; i++){
			fps[i] = fps[i+1];
		}

		fps[7] = 100000 / time;
		float sum = 0;
		for (i = 0; i < 8; i++) sum += fps[i];
		
		sprintf(str,"%d", (int) (sum / 8 + 0.5f));

	
		glColor3f(1,1,1);
		
		texHandler->setTextures(text);
		texHandler->setBlending(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		texHandler->apply();

		glDrawText(str, 0.02f, 0.02f, 0.03f, 0.03f);

		if (debug){
			glColor3f(1,1,1);
			texHandler->setTextures(renderTexture);
			texHandler->apply();
			
			glBegin(GL_QUADS);
				glTexCoord2f(0,1);
				glVertex2f(0.01f, 0.85f);
				glTexCoord2f(1,1);
				glVertex2f(0.14f, 0.85f);
				glTexCoord2f(1,0);
				glVertex2f(0.14f, 0.99f);
				glTexCoord2f(0,0);
				glVertex2f(0.01f, 0.99f);
			glEnd();

			texHandler->setTextures(renderTexture2);
			texHandler->apply();
			
			glBegin(GL_QUADS);
				glTexCoord2f(0,1);
				glVertex2f(0.16f, 0.85f);
				glTexCoord2f(1,1);
				glVertex2f(0.29f, 0.85f);
				glTexCoord2f(1,0);
				glVertex2f(0.29f, 0.99f);
				glTexCoord2f(0,0);
				glVertex2f(0.16f, 0.99f);
			glEnd();


			texHandler->setTextures(proceduralTexture);
			texHandler->apply();

			glBegin(GL_QUADS);
				glTexCoord2f(0,1);
				glVertex2f(0.85f, 0.8f);
				glTexCoord2f(1,1);
				glVertex2f(0.99f, 0.8f);
				glTexCoord2f(1,0);
				glVertex2f(0.99f, 0.99f);
				glTexCoord2f(0,0);
				glVertex2f(0.85f, 0.99f);
			glEnd();
		}
	}

	SwapBuffers(hdc);
}


OpenGLWindow::OpenGLWindow(){
	Width   = 1024;
	Height  = 768;
	Bpp     = 32;
	Refresh = 85;
	fullScreen = true;
	captureMouse = true;

	mouseSensibility = 0.1f;
	invertMouse = true;

	middleX = Width / 2;
	middleY = Height / 2;
	setResolution(Width, Height, Bpp, Refresh);

    SetCursorPos(middleX, middleY);
	
	debug = false;
    showFPS = true;

	position = Vertex(384,257,-383);
	
	Wx = 30;
	Wy = -48;
	Wz = 0;

	ZeroMemory(&Key, sizeof(Key));
	leftButton = false;

	CRect r(0, 0, Width, Height);
	windowedRect = r;

	Create(NULL, "OpenGL", WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, r);
}

BOOL OpenGLWindow::PreCreateWindow(CREATESTRUCT &cs){
	CFrameWnd::PreCreateWindow(cs);

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	return 1;
}


int OpenGLWindow::OnCreate(LPCREATESTRUCT lpCreateStruct){
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1) return -1;

	ShowWindow(SW_SHOW);
	SetForegroundWindow();
	SetFocus();

	hdc = ::GetDC((HWND) *this);

    static PIXELFORMATDESCRIPTOR pfd = {
        sizeof (PIXELFORMATDESCRIPTOR),
			1,
			PFD_DRAW_TO_WINDOW |
			PFD_SUPPORT_OPENGL |
			PFD_DOUBLEBUFFER,
			PFD_TYPE_RGBA,
			Bpp,
			0, 0, 0, 0, 0, 0,
			0, 0,
			0, 0, 0, 0, 0,
			Bpp == 32? 24:16,
			0,
			0,
			PFD_MAIN_PLANE,
			0,
			0, 0, 0
    };

    int nPixelFormat = ChoosePixelFormat(hdc, &pfd);
    SetPixelFormat(hdc, nPixelFormat, &pfd);

	hrc = wglCreateContext(hdc);
	wglMakeCurrent(hdc,hrc);
	
	ShowCursor(FALSE);

    initCPU();
    cpuHz = getHz();

	initExtensions(hdc);

    CString missing, supported;

    if (GL_ATI_fragment_shader)      supported += "GL_ATI_fragment_shader\n";    else missing += "GL_ATI_fragment_shader\n";
    if (GL_ARB_texture_env_dot3)     supported += "GL_ARB_texture_env_dot3\n";   else missing += "GL_ARB_texture_env_dot3\n";
    if (GL_ARB_texture_env_crossbar || GL_NV_texture_env_combine4)  supported += "GL_ARB_texture_env_crossbar/GL_NV_texture_env_combine4\n";  else missing += "GL_ARB_texture_env_crossbar/GL_NV_texture_env_combine4\n";
    if (GL_EXT_texture3D)            supported += "GL_EXT_texture3D\n";          else missing += "GL_EXT_texture3D\n";
    if (GL_EXT_texture_edge_clamp)   supported += "GL_EXT_texture_edge_clamp\n"; else missing += "GL_EXT_texture_edge_clamp\n";
    if (GL_SGIS_generate_mipmap)     supported += "GL_SGIS_generate_mipmap\n";   else missing += "GL_SGIS_generate_mipmap\n";
    if (WGL_ARB_render_texture)      supported += "WGL_ARB_render_texture\n";    else missing += "WGL_ARB_render_texture\n";

    if (missing.GetLength() > 0){
        MessageBox("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extension aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions", MB_OK);
        texHandler = NULL;
        PostMessage(WM_CLOSE);
        return 0;
    }

	texHandler = new TextureHandler();

    fragmentShader = texHandler->beginFragmentShader();
        // Pass 1
        glPassTexCoordATI(GL_REG_0_ATI, GL_TEXTURE0_ARB, GL_SWIZZLE_STQ_DQ_ATI);
        glSampleMapATI   (GL_REG_2_ATI, GL_TEXTURE2_ARB, GL_SWIZZLE_STR_ATI);

        GLfloat smallNumber[] = {0.035f, 0.035f, 0.00f, 0.0f};
        glSetFragmentShaderConstantATI(GL_CON_0_ATI, smallNumber);

        GLfloat blue[] = {0.2f, 0.5f, 1, 0.0f};
        glSetFragmentShaderConstantATI(GL_CON_1_ATI, blue);

        glColorFragmentOp3ATI(GL_MAD_ATI, GL_REG_1_ATI, GL_NONE, GL_NONE,
                                          GL_CON_0_ATI, GL_NONE, GL_NONE,
                                          GL_REG_2_ATI, GL_NONE, GL_BIAS_BIT_ATI,
                                          GL_REG_0_ATI, GL_NONE, GL_NONE);

        glColorFragmentOp3ATI(GL_MAD_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
                                          GL_CON_0_ATI, GL_NONE, GL_NONE,
                                          GL_REG_2_ATI, GL_NONE, GL_BIAS_BIT_ATI | GL_COMP_BIT_ATI,
                                          GL_REG_0_ATI, GL_NONE, GL_NONE);
        // Pass 2
        glSampleMapATI(GL_REG_0_ATI, GL_REG_0_ATI,    GL_SWIZZLE_STR_ATI);
        glSampleMapATI(GL_REG_1_ATI, GL_REG_1_ATI,    GL_SWIZZLE_STR_ATI);
        glSampleMapATI(GL_REG_2_ATI, GL_TEXTURE2_ARB, GL_SWIZZLE_STR_ATI);

        glColorFragmentOp2ATI(GL_MUL_ATI,  GL_REG_0_ATI, GL_NONE, GL_SATURATE_BIT_ATI,
                                           GL_REG_0_ATI, GL_NONE, GL_NONE,
                                           GL_CON_1_ATI, GL_NONE, GL_NONE);

        glColorFragmentOp3ATI(GL_LERP_ATI, GL_REG_0_ATI,         GL_NONE, GL_NONE,
                                           GL_PRIMARY_COLOR_ARB, GL_NONE, GL_NONE,
                                           GL_REG_0_ATI,         GL_NONE, GL_NONE,
                                           GL_REG_1_ATI,         GL_NONE, GL_NONE);

    texHandler->endFragmentShader(fragmentShader);



    vectorField    = texHandler->createVectorField(64);

	text           = texHandler->loadFromFile("..\\Textures\\font2.dds");
    renderTexture  = texHandler->createRenderTexture(512,512,false);
    renderTexture2 = texHandler->createRenderTexture(512,512,false);
	corona     = texHandler->loadFromFile("..\\Textures\\Corona2.png");
	
	wall       = texHandler->loadFromFile("..\\Textures\\floor4.dds");
	wallBump   = texHandler->loadFromFile("..\\Textures\\floor4Bump.png");
	texHandler->toNormalMap(wallBump);
	texHandler->setInternalFormat(wallBump, GL_RGB5);

	wood       = texHandler->loadFromFile("..\\Textures\\Wood.dds");
	woodBump   = texHandler->loadFromFile("..\\Textures\\WoodBump.png");
	texHandler->toNormalMap(woodBump);
	texHandler->setInternalFormat(woodBump, GL_RGB5);

	wood2      = texHandler->loadFromFile("..\\Textures\\wooddecoration1.dds");
	wood2Bump  = texHandler->loadFromFile("..\\Textures\\wooddecoration1Bump.png");
	texHandler->toNormalMap(wood2Bump);
	texHandler->setInternalFormat(wood2Bump, GL_RGB5);

	ground     = texHandler->loadFromFile("..\\Textures\\Fieldstone.dds");
	groundBump = texHandler->loadFromFile("..\\Textures\\FieldstoneBump.png");
	texHandler->toNormalMap(groundBump);
	texHandler->setInternalFormat(groundBump, GL_RGB5);

	picture     = texHandler->loadFromFile("..\\Textures\\picture3.dds");
	pictureBump = texHandler->loadFromFile("..\\Textures\\picture3Bump.png");
	texHandler->toNormalMap(pictureBump);
	texHandler->setInternalFormat(pictureBump, GL_RGB5);

    proceduralTexture = texHandler->createProceduralTexture(MakePerlinNoiseTexture);


	texHandler->uploadAll();

	// Roof
	polys.add(Poly(Vertex(-512,  640, -512), Vertex( 512,  640, -512), Vertex(-512,  640,  512), wood,   woodBump, 4, 4));

	// Walls
	polys.add(Poly(Vertex(-512,  640,  512), Vertex( 512,  640,  512), Vertex(-512,   -8,  512), wall,   wallBump, 8, 5));
	polys.add(Poly(Vertex( 512,  640,  512), Vertex( 512,  640, -512), Vertex( 512,   -8,  512), wall,   wallBump, 8, 5));

	polys.add(Poly(Vertex(-512,  640, -512), Vertex(-512,  640,  512), Vertex(-512,  512, -512), wall,   wallBump, 8, 1));
	polys.add(Poly(Vertex(-512,  128, -512), Vertex(-512,  128,  512), Vertex(-512,   -8, -512), wall,   wallBump, 8, 1));
	polys.add(Poly(Vertex(-512,  512, -512), Vertex(-512,  512, -384), Vertex(-512,  128, -512), wall,   wallBump, 1, 3));
	polys.add(Poly(Vertex(-512,  512, -128), Vertex(-512,  512,  128), Vertex(-512,  128, -128), wall,   wallBump, 2, 3));
	polys.add(Poly(Vertex(-512,  512,  384), Vertex(-512,  512,  512), Vertex(-512,  128,  384), wall,   wallBump, 1, 3));
	polys.add(Poly(Vertex(-512,  512, -384), Vertex(-512,  512, -128), Vertex(-512,  128, -384), picture,pictureBump, 1, 1));
	polys.add(Poly(Vertex(-512,  512,  128), Vertex(-512,  512,  384), Vertex(-512,  128,  128), picture,pictureBump, 1, 1));

	polys.add(Poly(Vertex( 512,  640, -512), Vertex(-512,  640, -512), Vertex( 512,  384, -512), wall,   wallBump, 8, 2));
	polys.add(Poly(Vertex( 512,  384, -512), Vertex( 128,  384, -512), Vertex( 512,   -8, -512), wall,   wallBump, 3, 4));
	polys.add(Poly(Vertex(-128,  384, -512), Vertex(-512,  384, -512), Vertex(-128,   -8, -512), wall,   wallBump, 3, 4));
	polys.add(Poly(Vertex( 128,  384, -512), Vertex(-128,  384, -512), Vertex( 128,   -8, -512), picture,pictureBump, 1, 1));

	// Box
	polys.add(Poly(Vertex( 512,   64,  512), Vertex( 512,   64,    0), Vertex( 256,   64,  512), wood2,  wood2Bump, 5, 3));
	polys.add(Poly(Vertex( 256,   64,  512), Vertex( 256,   64,    0), Vertex( 256,   -8,  512), wood2,  wood2Bump, 5, 1));
	polys.add(Poly(Vertex( 256,   64,    0), Vertex( 512,   64,    0), Vertex( 256,   -8,    0), wood2,  wood2Bump, 3, 1));

	// Walls
	polys.add(Poly(Vertex(-512,    8,  512), Vertex( 512,    8,  512), Vertex(-512, -128,  512), wall,   wallBump, 8, 1));
	polys.add(Poly(Vertex( 512,    8,  512), Vertex( 512,    8, -512), Vertex( 512, -128,  512), wall,   wallBump, 8, 1));
	polys.add(Poly(Vertex( 512,    8, -512), Vertex(-512,    8, -512), Vertex( 512, -128, -512), wall,   wallBump, 8, 1));
	polys.add(Poly(Vertex(-512,    8, -512), Vertex(-512,    8,  512), Vertex(-512, -128, -512), wall,   wallBump, 8, 1));

	// Box
	polys.add(Poly(Vertex( 256,    8,  512), Vertex( 256,    8,    8), Vertex( 256, -128,  512), wood2,  wood2Bump, 5, 2));
	polys.add(Poly(Vertex( 256,    8,    8), Vertex( 512,    8,    8), Vertex( 256, -128,    8), wood2,  wood2Bump, 3, 2));

	// Floor
	polys.add(Poly(Vertex(-512, -128,  512), Vertex( 256, -128,  512), Vertex(-512, -128, -512), ground, groundBump, 3, 4));
	polys.add(Poly(Vertex( 256, -128,    0), Vertex( 512, -128,    0), Vertex( 256, -128, -512), ground, groundBump, 1, 2));

	return 0;
}

void OpenGLWindow::OnSize(UINT nType, int cx, int cy){
	glViewport(0, 0, cx, cy);
}

void OpenGLWindow::OnClose(){
    if (texHandler != NULL){
	    texHandler->unloadAll();
	    delete texHandler;
    }

	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hrc);
	::ReleaseDC((HWND) *this,hdc);

	if (fullScreen) resetResolution();

	ShowCursor(TRUE);
	DestroyWindow();
}


void OpenGLWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags){
	Key[nChar & 0xFF] = true;

    switch(nChar){
        case ' ':
            showFPS = !showFPS;
            break;
        case 'D':
            debug = !debug;
            break;
        case VK_ESCAPE:
            if (!fullScreen && captureMouse){
                ShowCursor(TRUE);
                captureMouse = false;
            } else {
                PostMessage(WM_CLOSE, 0,0);
            }
            break;
    }
}

void OpenGLWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags){
	Key[nChar & 0xFF] = false;
}

void OpenGLWindow::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags){
	if (((nFlags >> 13) & 0x3) == 1 && nChar == VK_ENTER){
		if (fullScreen){
			resetResolution();
			middleX = (windowedRect.left + windowedRect.right)  / 2;
			middleY = (windowedRect.top  + windowedRect.bottom) / 2;
			MoveWindow(windowedRect, TRUE);

			fullScreen = false;
		} else {
			middleX = Width  / 2;
			middleY = Height / 2;
			MoveWindow(0, 0, Width, Height, TRUE);
			setResolution(Width, Height, Bpp, Refresh);

			captureMouse = true;
			ShowCursor(FALSE);

			fullScreen = true;
		}
	}
}

void OpenGLWindow::OnLButtonDown(UINT nFlags, CPoint point){
	mouseX = point.x;
	mouseY = point.y;

	windowMoved = false;
	leftButton = true;
}

void OpenGLWindow::OnLButtonUp(UINT nFlags, CPoint point){
	if (!captureMouse){
		if (!windowMoved){
			captureMouse = true;
			ShowCursor(FALSE);
		}
	}

	leftButton = false;
}

void OpenGLWindow::OnMouseMove(UINT nFlags, CPoint point){
	static bool changeMouse = false;

	if (captureMouse){
		ClientToScreen(&point);

		Wx += mouseSensibility * (point.y - middleY) * (invertMouse? -1 : 1);
		Wy += mouseSensibility * (point.x - middleX);
		
		if (changeMouse = !changeMouse) SetCursorPos(middleX, middleY);
	} else {
		if (leftButton){
			windowedRect += CSize(point.x - mouseX, point.y - mouseY);
			middleX = (windowedRect.left + windowedRect.right)  / 2;
			middleY = (windowedRect.top  + windowedRect.bottom) / 2;

			MoveWindow(windowedRect, TRUE);
			windowMoved = true;
		}
	}
}

void OpenGLWindow::OnKillFocus(CWnd *pNewWnd){
	if (fullScreen){
		resetResolution();
		middleX = (windowedRect.left + windowedRect.right)  / 2;
		middleY = (windowedRect.top  + windowedRect.bottom) / 2;
		MoveWindow(windowedRect, TRUE);
		fullScreen = false;
		captureMouse = false;
		ShowCursor(TRUE);
	}
}

/* ################################################################################################################### */


class OpenGLApp : public CWinApp {
public:
    virtual BOOL InitInstance();
	BOOL OnIdle(LONG lCount);
};


BOOL OpenGLApp::InitInstance(){
	m_pMainWnd = new OpenGLWindow();

	return TRUE;
}

BOOL OpenGLApp::OnIdle(LONG lCount){
	
	((OpenGLWindow *) m_pMainWnd)->DrawFrame();

	return TRUE;
}

OpenGLApp openGLApp;


/* ########################################## End of file ############################################################ */
